@@ -1,9 +1,8 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2012 Lance Edgar
+#  Copyright © 2010-2014 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -36,10 +35,9 @@ import logging
 import datetime
 from decimal import Decimal
 
-from sqlalchemy.orm.exc import NoResultFound
+import pytz
 
-import edbob
-from edbob.time import localize
+from sqlalchemy.orm.exc import NoResultFound
 
 from rattail.core import Object
 from rattail.gpc import GPC
@@ -48,7 +46,9 @@ from rattail.db import model
 from rattail.db.api.products import get_product_by_upc
 from rattail import enum
 from rattail.util import load_object
+from rattail.filemon import Action
 from rattail.files import overwriting_move
+from rattail.time import localtime
 
 
 change_pattern_generic = re.compile(r'^(?P<batch_id>\d{8})_(?P<batch_name>.+)$')
@@ -57,12 +57,15 @@ change_pattern = re.compile(r'^\d{8}_(?P<batch_name>.+)\.csv$', re.IGNORECASE)
 log = logging.getLogger(__name__)
 
 
-class ChangesProcessor(Object):
+class ChangesProcessor(object):
     """
     Base class and default implementation for processing changes which have
     been deployed from SMS.
     """
 
+    def __init__(self, config):
+        self.config = config
+
     def process_all_changes(self, changes):
         session = Session()
 
@@ -126,12 +129,20 @@ class ChangesProcessor(Object):
         return int(val) if val else None
 
     def utc_timestamp(self, date, time):
-        if not date:
+        """
+        Return a datetime which combines the two parameters, and is localized
+        to the UTC time zone, but which is still naive.  The idea here is to
+        obtain a value which may be compared directly to those which are stored
+        within the Rattail database, since those are naive UTC as well.
+        """
+        if date is None:
             return None
-        if not time:
+        if time is None:
             time = datetime.time(0) # assume midnight
-        stamp = datetime.datetime.combine(date, time)
-        return localize(stamp, from_='rattail.sw.locsms', to='utc', naive=True)
+        time = datetime.datetime.combine(date, time)
+        time = localtime(self.config, time, 'rattail.sw.locsms')
+        time = time.astimezone(pytz.utc)
+        return time.replace(tzinfo=None)
 
     def get_product(self, upc, session):
         upc = GPC(upc, calc_check_digit='upc')
@@ -190,8 +201,20 @@ class ChangesProcessor(Object):
         
         return p
 
+    def utcnow(self):
+        """
+        Return the current naive UTC date and time.  (This is abstracted mostly
+        for the convenience of tests.)
+        """
+        return datetime.datetime.utcnow()
+
     def price_is_current(self, price):
-        now = edbob.utc_time(naive=True)
+        """
+        Determine if the given :class:`rattail.db.model.ProductPrice` instance
+        should be considered "current", i.e. if the current date and time would
+        fall "inside" the price's start and/or end datetimes.
+        """
+        now = self.utcnow()
         if price.starts and price.ends:
             return price.starts <= now <= price.ends
         if price.starts:
@@ -881,12 +904,12 @@ class ChangesProcessor(Object):
         session.delete(employee)
 
 
-def process_changes(path, backup_dir='processed', processor_factory=None):
+class ProcessChanges(Action):
     """
     Processes "changes" CSV data which has been deployed from an SMS server
     instance, with the Deploy_Rattail option installed.
 
-    This function is designed to be called by the file monitor.  An example
+    This action is designed to be called by the file monitor.  An example
     configuration would be (note that some things are optional):
 
     .. code-block:: ini
@@ -895,54 +918,62 @@ def process_changes(path, backup_dir='processed', processor_factory=None):
        monitor = sms_changes
        sms_changes.dirs = C:\Storeman\XchRattail\Changes
        sms_changes.actions = process
-       sms_changes.action.process.func = rattail_locsms.changes:process_changes
-       sms_changes.action.process.kwarg.backup_dir = /my/changes/backup/dir
+       sms_changes.action.process.class = rattail_locsms.changes:ProcessChanges
        sms_changes.action.process.kwarg.processor_factory = mymodule:MyChangesProcessor
-
-    If you wish to override the default data processor class, then you should
-    additionally provide configuration similar to this:
-
-    .. code-block:: ini
-
-       [rattail.sw.locsms]
-       changes.processor_class = myrattail.mymodule:MyChangesProcessor
+       sms_changes.action.process.kwarg.backup_dir = C:\Storeman\XchRattail\Changes\Backup
     """
-    match = change_pattern_generic.match(os.path.basename(path))
-    if not match:
-        log.warning("unexpected filename pattern: {0}".format(repr(path)))
-        return
 
-    # Ignore all incoming files until we see one indicating "batch" completion.
-    if match.group('batch_name').upper() != 'CHANGES_END':
-        return
+    def __call__(self, path, backup_dir='processed', processor_factory=None):
+        """
+        Process a change file.
 
-    deploy_id = match.group('batch_id')
+        Note that in reality, the only time processing will occur is when the
+        final "CHANGES_END" file is seen.  I.e. this method will merely ignore
+        all files until the "end" file, at which point all files deemed part of
+        the current "batch" will be processed at once.
 
-    changes = {}
-    changes_dir = os.path.dirname(path)
-    for fn in os.listdir(changes_dir):
-        if fn.startswith(deploy_id):
-            match = change_pattern.match(fn)
-            if match:
-                changes[match.group('batch_name').upper()] = os.path.join(changes_dir, fn)
+        If no custom ``processor_factory`` is provided, it will default to the
+        :class:`ChangesProcessor` class.
 
-    if processor_factory is not None:
-        log.debug("using custom changes processor factory: {0}".format(processor_factory))
-        processor_factory = load_object(processor_factory)
-    else:
-        processor_factory = ChangesProcessor
+        Once processing is complete, all files from the current batch will be
+        moved to the ``backup_dir`` location.
+        """
+        match = change_pattern_generic.match(os.path.basename(path))
+        if not match:
+            log.warning("unexpected filename pattern: {0}".format(repr(path)))
+            return
 
-    processor = processor_factory()
-    processor.process_all_changes(changes)
+        # Ignore all incoming files until we see one indicating "batch" completion.
+        if match.group('batch_name').upper() != 'CHANGES_END':
+            return
 
-    if os.path.basename(backup_dir) == backup_dir:
-        backup_dir = os.path.join(changes_dir, backup_dir)
-    else:
-        backup_dir = os.path.abspath(backup_dir)
+        deploy_id = match.group('batch_id')
 
-    log.debug("backing up data files in dir: {0}".format(backup_dir))
-    if not os.path.exists(backup_dir):
-        os.makedirs(backup_dir)
-    for fn in list(os.listdir(changes_dir)):
-        if fn.startswith(deploy_id):
-            overwriting_move(os.path.join(changes_dir, fn), backup_dir)
+        changes = {}
+        changes_dir = os.path.dirname(path)
+        for fn in os.listdir(changes_dir):
+            if fn.startswith(deploy_id):
+                match = change_pattern.match(fn)
+                if match:
+                    changes[match.group('batch_name').upper()] = os.path.join(changes_dir, fn)
+
+        if processor_factory is not None:
+            log.debug("using custom changes processor factory: {0}".format(processor_factory))
+            processor_factory = load_object(processor_factory)
+        else:
+            processor_factory = ChangesProcessor
+
+        processor = processor_factory(self.config)
+        processor.process_all_changes(changes)
+
+        if os.path.basename(backup_dir) == backup_dir:
+            backup_dir = os.path.join(changes_dir, backup_dir)
+        else:
+            backup_dir = os.path.abspath(backup_dir)
+
+        log.debug("backing up data files in dir: {0}".format(backup_dir))
+        if not os.path.exists(backup_dir):
+            os.makedirs(backup_dir)
+        for fn in list(os.listdir(changes_dir)):
+            if fn.startswith(deploy_id):
+                overwriting_move(os.path.join(changes_dir, fn), backup_dir)
