@@ -30,20 +30,20 @@ from __future__ import unicode_literals
 import datetime
 import logging
 
+import pytz
+
 from sqlalchemy.orm import joinedload_all
 
 from locsms.db import Session as SmsSession
 from locsms.db import model as locsms
 
-import edbob
-from edbob.time import localize
-
 from rattail.gpc import GPC
 from rattail.db import Session
 from rattail.db import model
 from rattail.db import cache
 from rattail import enum
 from rattail.db.api import get_setting, save_setting
+from rattail.time import localtime
 
 
 log = logging.getLogger(__name__)
@@ -423,7 +423,11 @@ class PriceLoader(DataLoader):
         yield cache.ProductCacher, 'products'
 
     def start(self):
-        self.now = edbob.utc_time(naive=True)
+        """
+        Cache the current naive UTC time, for use in determining whether or not
+        prices should be considered current.
+        """
+        self.now = datetime.datetime.utcnow()
 
     def get_sms_query(self):
         q = super(PriceLoader, self).get_sms_query()
@@ -431,14 +435,22 @@ class PriceLoader(DataLoader):
         return q
 
     def utc_timestamp(self, date, time):
-        if not date:
+        """
+        Return a datetime which combines the two parameters, and is localized
+        to the UTC time zone, but which is still naive.  The idea here is to
+        obtain a value which may be compared directly to those which are stored
+        within the Rattail database, since those are naive UTC as well.
+        """
+        if date is None:
             return None
-        if time:
-            time = datetime.datetime.strptime(time, '%H%M').time()
-        else:
+        if time is None:
             time = datetime.time(0) # assume midnight
-        stamp = datetime.datetime.combine(date, time)
-        return localize(stamp, from_='rattail.sw.locsms', to='utc', naive=True)
+        else:
+            time = datetime.datetime.strptime(time, '%H%M').time()
+        time = datetime.datetime.combine(date, time)
+        time = localtime(self.config, time, 'rattail.sw.locsms')
+        time = time.astimezone(pytz.utc)
+        return time.replace(tzinfo=None)
 
     def price_is_current(self, price):
         if price.starts and price.ends:
